# -*- coding: latin-1 -*-
'''
checkIGS.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.exceptions.dontRunException import dontRunException

import urllib2
import re

class checkIGS(basePlugin):
	'''
	Performs audit of SAP Internet Graphics Service (IGS), if detected.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._iface = None
		self._targets = {}
		self._vulnVersions = []
		self._host = 'Current target'
		self._ports = [8030, 40080]	
		self._proto = 'http'
		
	def run( self ):
		try:
			core.sCore.validateTargets(self._targets)
		except dontRunException:
			return False		

		if self._host == 'Current target':
			igshost = self._conn.ashost
		else:
			igshost = self._host
		
		for port in self._ports:
			try:
				# Connect to the HTTP service
				url = self._proto + '://' + igshost + ':' + str(port)
				res = urllib2.urlopen(url)
				body = res.read()
				
				om.out.information('\tConnected to SAP IGS HTTP interface.')
				kb.kb.save('checkIGS', 'status', 1)
				# We check for the title, to see if Administration is enabled.
				if re.compile('SAP IGS Administration').search(body):
					# Administration enabled
					# Now we check to see if it is vulnerable to reported bugs. See:
					# http://www.cybsec.com/vuln/CYBSEC-Security_Pre-Advisory_SAP_IGS_Undocumented_Features.pdf
					# http://www.cybsec.com/vuln/CYBSEC-Security_Pre-Advisory_SAP_IGS_Remote_Arbitrary_File_Removal.pdf
					# http://www.cybsec.com/vuln/CYBSEC-Security_Pre-Advisory_SAP_IGS_Remote_Buffer_Overflow.pdf
					# http://www.cybsec.com/vuln/CYBSEC-Security_Pre-Advisory_SAP_IGS_Remote_Denial_of_Service.pdf
					# http://research.corsaire.com/advisories/c050503-001.txt
					
					# Just based on version reported. :(
					# TODO: Upgrade to verify through availability of commands.
					# TODO: Add reportedVuln model!!
					version = re.findall('(\d{4}\.\d*\.\d*\.\d*)', body)[0]
					rel = version[0]
					patchl = int(version.split('.')[2])
					om.out.information('\tSAP IGS reported version: ' + version)
					om.out.information('\tHTTP Administration is enabled, accessible at ' + url)
					if (rel == '6' and patchl <= 16) or (rel == '7' and patchl <= 6):
						om.out.information('\tThis version has reported vulnerabilities. Check them in security sites!')
						kb.kb.save('checkIGS', 'status', 2)
				else:
					# Administration disabled
					om.out.information('\tHTTP Administration is disabled.')
				
				return True
			except urllib2.URLError:
				om.out.information('\tIGS not available at ' + url)
				# status: 0 = down, 1 = up but not vuln, 2 = vuln
				kb.kb.save('checkIGS', 'status', 0)
			
		return True

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="host">\
				<default>Current target</default>\
				<desc>SAP IGS host</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="ports">\
				<default>8030,40080</default>\
				<desc>IGS HTTP port</desc>\
				<type>NumericList</type>\
				<required>True</required>\
			</Option>\
			<Option name="protocol">\
				<default>http</default>\
				<desc>Protocol for connection (http | https)</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'host' in OptionList.keys(): 
			self._host = OptionList['host']
		if 'ports' in OptionList.keys(): 
			self._ports = OptionList['ports']
		if 'protocol' in OptionList.keys(): 
			self._proto = OptionList['protocol']

